#!/usr/bin/env perl
use strict;

=head1 LICENSE

   Bacula - The Network Backup Solution

   Copyright (C) 2001-2023 Bacula Systems SA
   All rights reserved.

   The main author of Bacula is Kern Sibbald, with contributions from many
   others, a complete list can be found in the file AUTHORS.

   Licensees holding a valid Bacula Systems SA license may use this file
   and others of this release in accordance with the proprietary license
   agreement provided in the LICENSE file.  Redistribution of any part of
   this release is not permitted.

   Bacula is a registered trademark of Kern Sibbald.

=head1 NAME

    bsys_report.pl -- Bacula System configuration reporting tool

=head1 SYNOPIS

    bsys_report.pl [-d /path/to/bacula-bin -d /path/to/bacula-conf...] [-b n]

=head2 DESCRIPTION 

    Bacula Systems provides this tool to their partners and customers to
    speed up issue reporting.
    
    bsys_report.pl includes configuration files in its report. It does replace
    passwords it finds in "Password=" lines (case insensitive), but it does
    NOT recognize and replace passwords you might use in database dump scripts,
    mail programs, or anywhere else!
    
    To be most effective, the following tools are needed:
    - uname  (should be available on all unix and linux systems)
    - mount  (ditto)
    - free   (ditto)
    - sar    (may not be installed on every system)
    - df     (should be available on all unix and linux systems)
    - lvm    (may not be installed on every system)
    - lspci  (may not be installed on every system)
    - lsscsi (ditto)
    - locate (may not be installed on every system and needs a current database)
    - tail   (should be available on all unix and linux systems)
    - file   (ditto)
    - md5sum (ditto)
    - ldd    (ditto)
    - dpkg or rpm
    - mysql or psql

=head2 EXAMPLE

   bsys_report.pl

   bsys_report.pl -d /home/bacula

   # If your bconsole has multiple directors
   bsys_report.pl -b 1

   # If you want to include other files or directories
   bsys_report.pl -i /opt/bacula/conf.d/test.conf
   bsys_report.pl -d /opt/bacula/conf.d/ -d /opt/bacula/other.d/

   # If you want to save the bsys_report elsewhere
   bsys_report.pl -o /some/different/folder

   # If you do not want to fetch any storage status information
   bsys_report.pl -s

   # If you do not want that bsys report uses bconsole to fetch any running information. Please notice that this also implies "-s" above
   bsys_report.pl -n

=head2 DEPENDENCIES

  Please notice that this script needs these perl extensions:

  English
  Getopt::Long
  Pod::Usage
  File::stat
  File::Basename
  POSIX

=cut

use English;
use Getopt::Long qw/:config no_ignore_case/;
use Pod::Usage;
use File::stat;
use File::Basename;
use POSIX qw(strftime);

sub exec_it
{
    my ($prog, $msg) = @_;
    if (system($prog) != 0) {
        print $msg;
        exit 1;
    }
}

my $bsysVer = "3.52";
my $loglines = 3000;
my $daysback = 90; # The number of days to look back for active clients
# Path where we search binaries and configuration files
my @rep = (
#BEE
	    qw(
               /opt/bacula /opt/bacula/etc /opt/bacula/bin /opt/bacula/sbin
               /opt/bacula/var /opt/bacula/working /opt/bacula/scripts ),
#/BEE
	    qw(
               /usr/local/bacula /usr/local/bacula/etc /usr/local/bacula/bin
               /usr/local/bacula/sbin /usr/local/bacula/var 
               /usr/local/bacula/working /usr/local/bacula/scripts
               /etc/bacula /usr/bin /usr/sbin /bacula /bacula/etc
               /bacula/bin /bacula/sbin /bacula/working /bacula/scripts
               )
    );
my @file_u;
my @rep_u;
# we run as bs_report
#my $d = `date +%Y-%m-%d`; chop($d);
my $d = strftime "%Y-%m-%d", localtime;
my $now_string;
my $hostname = `hostname`;
chomp($hostname);
$hostname =~ s/[^\d\w\.\-]//g;
my $dir_num=1;
my $nodirinfo = 0;
my $skip_storage='';
my $findopt='-xdev';
# This is the default path for the community
my $bsys_path;
my $arch_name;
my $bsys_out = '';

my ($help);
my @ARGV_bkp = @ARGV;
my $FPTMP;
GetOptions('help'     => \$help,
           'dir=s'    => \@rep_u,
           'include=s' => \@file_u,
           'bconsole=i' => \$dir_num,
	   'skip_storage' => \$skip_storage,
	   'nodirinfo' => \$nodirinfo,
	   'output=s' => \$bsys_path )
    || Pod::Usage::pod2usage(-exitval => 2, -verbose => 2) ;

# look user directories first
unshift @rep, @rep_u;

if ($help) {
    Pod::Usage::pod2usage(-exitval => 2, -verbose => 2) ;
}

if ($bsys_path eq "" ) {
  $bsys_out="/tmp";
#BEE
  $bsys_out="/opt/bacula/working";
#/BEE
}
else {
  if ( $bsys_path ne "-" ) {
    $bsys_out = $bsys_path;
  }
}

$arch_name="$bsys_out/bsys_report.$d.$hostname.$$.gz";


my %pass;                       # password list
my $p=0;

my @conf = ( qw( bacula-dir.conf bacula-sd.conf bconsole.conf
                 bacula-fd.conf ) ) ;
# optional configuration files
my @opt_conf = ( qw( mtx-changer mtx-changer.conf bat.conf bacula-tray-monitor.conf ),
#BEE
		  qw(
                 vsphere_global.conf ldap.conf msad.conf bsnapshot.conf
               )
#/BEE
	       );
my @bin  = ( qw( bacula-dir bacula-sd bacula-fd bconsole bacula_config ) );
my %include;                    # get included files
my $fst;
#BEE
my $vsphereplugin = 0;
my $hypervplugin = 0;
my $xenserverplugin = 0;
my $rhvplugin = 0;
my $proxmoxplugin = 0;
my $kvmplugin = 0;
my $nutanixplugin = 0;
my $qemuplugin = 0;
my $azureplugin = 0;
my $ec2plugin = 0;
my $sanplugin = 0;
my @dedupfolders;
#/BEE
my @logfolders = ();
my @logs = (qw( /var/log/messages /var/log/kern.log /var/log/kernel.log 
                /var/log/syslog /var/log/syslog.log /var/adm/messages ),
#BEE
            qw(
                /opt/bacula/working/bweb-error.log
                /opt/bacula/bin/vsphere-ctl.log
                /opt/bacula/working/vsphere-ctl.log
              )
#/BEE
);

my @osbanners = ( qw( /etc/redhat-release /etc/debian_version /etc/lsb-release /etc/slackware-version /etc/SuSE-release /etc/oracle-release ) );

$ENV{LANG}='C';

if ( $bsys_out ne "" ) {
  print "Dumping result to $arch_name, attach this file to your support issue\n\n";
  close(STDOUT);
  open(STDOUT, "|gzip -c > $arch_name")
    or die "ERR: Can't open $arch_name for output";
  select(STDOUT);
}

#$now_string = strftime( "%a %b %e %H:%M:%S %Y", localtime );
print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering general server information.\n";
print   "#============== bs_gen_status ====================================#\n";
print "Generated with bsys_report version $bsysVer on ";
system("date");
print "\n#============== bs_gen_arg =======================================#\n";
print "Arguments: ", join(" ", @ARGV_bkp);
print "\n#============== bs_sw_os =========================================#\n";
print "Sysinfo\n";
system("uname -a");
print "\n";
foreach my $f (@osbanners) {
  if ( -f $f ) {
    strip_conf( $f );
  }
}
system("uptime");
print "\n#============== bs_sw_lsmod ======================================#\n";
print "\nlsmod:\n";
system("lsmod");
print "\n#============== bs_hw_network === ifconfig =======================#\n";
system( "ifconfig -a || ip -s addr ls" );
print "\n#============== bs_hw_mounted_partitions === mount =======================#\n";
print "\nPartitions:\n";
system("mount");
if ($^O ne 'linux') {
    print "\n#============== bs_hw_disk_usage === df ==========================#\n";
    print "\nDisk Usage:\n";
    system("df -k");
    print "\n#============== bs_hw_memory === free ============================#\n";
    print "\nMemory:\n";
    system("free");
}

if ($^O eq 'linux') {
    print "\n#============== bs_hw_disk_usage === df ==========================#\n";
    print "\nDisk Usage:\n";
    system("df -h");
    print "\n#============== bs_hw_disk_mdstat === mdstat =====================#\n";
    print "\nsoftRAID configuration:\n";
    strip_conf("/proc/mdstat");
    print "\n#============== bs_hw_pd_partition === lsblk =====================#\n";
    print "\nPhysical disk partitions:\n";
    system("lsblk -e 7 -o NAME,MAJ:MIN,RM,SIZE,RO,TYPE,ROTA,MOUNTPOINT");
    print "\n#============== bs_hw_pv_usage === pvs ===========================#\n";
    print "\nLVM Physical Volume configuration:\n";
    system("pvs");
    print "\n#============== bs_hw_vg_usage === vgs ===========================#\n";
    print "\nLVM VG configuration:\n";
    system("vgs");
    print "\n#============== bs_hw_lv_usage === lvs ===========================#\n";
    print "\nLVM LV configuration:\n";
    system("lvs");
    print "\n#============== bs_hw_memory === free ============================#\n";
    print "\nMemory:\n";
    if ( system("free -h") ) {
       print "No human-readable flag support!!\n";
       system( "free" );
    }
    print "\n#============== bs_hw_lsscsi =====================================#\n";
    print "\nlsscsi:\n";
    system("lsscsi -gdvlL 2>/dev/null") == 0 or system("[ -f /proc/scsi/scsi ] && cat /proc/scsi/scsi");
    system("lsscsi -Lt 2>/dev/null") == 0 or system("[ -f /proc/scsi/scsi ] && cat /proc/scsi/scsi");
    print "\n#============== bs_hw_udev =======================================#\n";
    if (-d "/dev/tape" ) {
      system("ls -lsaR /dev/tape/");
    }
    # lsscsi -gdvlL
    #system("ls -altrh /dev/*/*/* | grep 's[tg]'");
    system("find /dev -name '*s[ctg]*' -ls");

    print "\n#============== bs_hw_pci === lspci ==============================#\n";
    print "\nPCI:\n";
    system("lspci");
    print "\n#============== bs_hw_lshw =======================================#\n";
    print "\nHardware CPU info:\n";
    strip_conf( "/proc/1/environ" );
    system("lshw -class system");
    print "\n#============== bs_hw_cpu ========================================#\n";
    print "\ncpuinfo:\n";
    strip_conf("/proc/cpuinfo");
    print "\n#============== bs_sw_packages === dpkg = rpm ====================#\n";
    print "\nPackages:\n";
    system("dpkg -l 2> /dev/null");
#    print "\n";
 #   my @debpkgs = `dpkg-query -W -f='\${Status}:\${binary:Package}\n' 'bacula-enterprise*'|grep 'ok installed:'|cut -f2 -d: 2>/dev/null`;
 #   foreach my $file (@debpkgs) {
 #       chomp($file);
 #       system("debsums -as $file 2>/dev/null")
 #   }
#    print "\n#============== bs_sw_packages ==================================#\n";
  if ( ( -x "/usr/bin/rpm" ) || ( -x "/bin/rpm" ) ) {
    system("rpm -qa 2> /dev/null");
    print "\n";
    my @rpmpkgs = `rpm -qa|grep bacula-enterprise 2>/dev/null`;
    foreach my $file (@rpmpkgs) {
        chomp($file);
        system("rpm --verify $file 2>/dev/null");
    }
  }
    print "\n#============== bs_sw_firewall === iptables ======================#\n";
    if ( -x "/usr/bin/firewall-cmd" ) {
        system("firewall-cmd --list-ports");
    } else {
        system("iptables -L 2> /dev/null");
        system("nft list ruleset");
    }
    print "\n#============== bs_sw_selinux ====================================#\n";
    system("getenforce 2> /dev/null");
} elsif ($^O eq 'solaris' or $^O eq 'sunos') {
    print "\n#============== bs_hw_lsscsi =====================================#\n";
    print "\nprtconf\n:";
    system("prtconf 2> /dev/null");
    print "\n#============== bs_hw_cpu ========================================#\n";
    print "\ncpuinfo:\n";
    system("psrinfo -vp 2> /dev/null");
    print "\n#============== bs_sw_packages === pkginfo =======================#\n";
    print "\nPackages:\n";
    system("pkginfo -x 2> /dev/null");
}

print "\n#============== bs_sw_sysctl =====================================#\n";
print "\nsysctl\n";
system("sysctl -a 2>/dev/null");
print "\n#============== bs_sw_uid ========================================#\n";
system("id bacula");
print "\n#============== bs_sw_sysproc ====================================#\n";
system("ps -ef|grep bacula-|grep -v grep");
system("ps -ef|grep bweb|grep -v grep");
system("ps -ef|grep light|grep -v grep");
system("ps -ef|grep httpd|grep -v grep");
system("ps -ef|grep apache|grep -v grep");

# system("ps -lfC bacula-dir");
# system("ps -lfC bacula-sd");
# system("ps -lfC bacula-fd");
print "\n#============== bs_sw_getent =====================================#\n";
system("getent services | grep 'bacula.*tcp'");
print "\n#============== bs_sw_hosts_access ===============================#\n";
system("egrep 'bacula|ALL:' /etc/hosts.*");
print "\n#============== bs_sw_limits =====================================#\n";
print "Current user limits:\n";
system("ulimit -a");
system("ps -ef|grep bacula-|grep -v grep|while read u1 pid n m d t h pro op;do echo \$pro':';[ -f /proc/\$pid/limits ] && cat /proc/\$pid/limits; done");

print "\n#============== bs_sw_socklisten =================================#\n";
system("(netstat -anp || ss -anp || netstat -a) | grep bacula");
system("(netstat -anp || ss -anp || netstat -a) | grep http");

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula configuration files.\n";

print "\n#============== bs_conf ==========================================#\n";
sub dump_conf 
{
    foreach my $f (keys %include) {
        my @fst;
        my @grinfo;

        if (-r $f) {
	    $fst = stat($f);
	    my ($fgname, $fgpasswd, $fggid, $fgmembers) = getgrgid( $fst->gid );

            print "\n#================================================================#\n";
            print "Content $f - (". strftime( "%F %T", localtime( $fst->mtime ) );
            printf (", %04o - %s:", $fst->mode & 07777, getpwuid($fst->uid) );
	    print "$fgname):\n";
#	    $grinfo = getgrgid( $fst->gid );
#            printf (", %04o - %s:%s):\n", $fst->mode & 07777, getpwuid($fst->uid), $grinfo->name );
            strip_conf($f);
        } else {
            print "\n#================================================================#\n";
            print "Content $f:\n";
            print "W: Cannot dump the content of $f, might be a program\n";
            print STDERR "Cannot dump the content of $f, might be a program\n";
        }
        delete $include{$f};
    }
}

sub getStorages {
  my %device;
  foreach my $var ( @_ ) {
    if ( $var =~ /^(Autochanger:|Storage:) name=(.+?) address=(.+?) SDport=(.+?) /i ) {
      $device{$3.$4}=$2 if not exists $device{$3.$4};
    }
  }
  
  return values %device;
}


my %already_seen;

# take user argument in account to dump configuration
foreach my $item (@file_u) {
    if (-d $item) {
        unshift @rep, $item;

    } elsif (-f $item) {
        $include{$item}=1;
    }
}

foreach my $f (@conf) {
    my $ok=0;
    my @fst;
    my @grinfo;
    foreach my $d (@rep) {
        if (-r "$d/$f" && !$already_seen{"$d/$f"}) {
            $already_seen{"$d/$f"}=1;
            $fst = stat( "$d/$f" );
	    my ($fgname, $fgpasswd, $fggid, $fgmembers) = getgrgid( $fst->gid );
            print "\n#================================================================#\n";
            print "Content $d/$f - (". strftime( "%F %T", localtime( $fst->mtime ) );
            printf (", %04o - %s:", $fst->mode & 07777, getpwuid($fst->uid) );
	    print "$fgname):\n";
#	    $grinfo = getgrgid( $fst->gid );
#            printf (", %04o - %s:%s):\n", $fst->mode & 07777, getpwuid($fst->uid), $grinfo->name );
            strip_conf("$d/$f");
            $ok++;
        }
    }
    if (!$ok) {
        print STDERR "Can't find $f, maybe a permission problem\n";
        print "W: Can't find $f, maybe a permission problem\n";
        print "W: Could be one of them:\n";
        system("locate $f 2>/dev/null");
    }
    if ($ok > 1) {
        print STDERR "Found multiple instances of $f\n";
        print "W: Found multiple instances of $f\n";
    }
}

# we strip all password, but we keep correspondences
sub strip_conf
{
    my ($file) = @_;
    my $FP;
    if (!open($FP, $file)) {
        print "Can't open $file $!\n";
        return 0;
    }

    # strip password
    while (my $l = <$FP>)
    {
	if ( ($l =~ /(password).*=>\s*['](.+?)[']([\s,;#]|$)/i )
	    or
	    ($l =~ /(pass|password|bucketname|secretkey|accesskey|secret_key|access_key)\s*=\s*["']*(.+?)["']*([\s;#]|$)/i) ) {
	      
# ($l =~ /(pass|password|bucketname|secretkey|accesskey)\s*=\s*"(.+?)"\s*([;#]|$)/i )
#	    or
#	    ($l =~ /\'password\'\s*=>\s*'(.+?)'\s*([;#]|$)/i )
#	    or
#	    ($l =~ /(pass|password|bucketname|secretkey|accesskey)\s*=\s*(.+?)["\s].*([;#]|$)/i) ) {
            my $oldp=$2;
            if (!defined $pass{$oldp}) {
                $pass{$oldp}="pass" . $p++;
            }

            $l =~ s/\Q$oldp/$pass{$oldp}/;   # use \Q to avoid $oldp => pattern
        }
        print $l;
#BEE
        if ( $l =~ /plugin.*vsphere/i ) {
          $vsphereplugin = 1;
        }
        if ( $l =~ /plugin.*hyperv/i ) {
          $hypervplugin = 1;
        }
        if ( $l =~ /plugin.*rhv/i ) {
          $rhvplugin = 1;
        }
        if ( $l =~ /plugin.*proxmox/i ) {
          $proxmoxplugin = 1;
        }
         if ( $l =~ /plugin.*xenserver/i ) {
          $xenserverplugin = 1;
        }
        if ( $l =~ /plugin.*kvm/i ) {
          $kvmplugin = 1;
        }
         if ( $l =~ /plugin.*nutanix-ahv/i ) {
          $nutanixplugin = 1;
        }
        if ( $l =~ /plugin.*qemu/i ) {
          $qemuplugin = 1;
        }
        if ( $l =~ /plugin.*azure-vm/i ) {
          $azureplugin = 1;
        }
        if ( $l =~ /plugin.*amazon-ec2/i ) {
          $ec2plugin = 1;
        }
        if ($l =~ /(dedupdirectory|dedupindexdirectory)\s*=\s*"(.+?)"/i) {
            push @dedupfolders, $2;
        }
        if ( $l =~ /lock.*device/i ) {
          $sanplugin = 1;
        }

#/BEE
        if ($l =~ /^\s*\@\|"(.+)"/) { # command output
            $l = $1;
            $l =~ s/\\"/"/g;
            strip_conf("$l|");

        } elsif ($l =~ /^\s*\@"\|(.+)"/) { # command output
            $l = $1;
            $l =~ s/\\"/"/g;
            strip_conf("$l|");

        } elsif ($l =~ /^\s*\@"([^|].+?)"/) { # normal files
            $include{$1}=1;

        } elsif ($l =~ /^\s*\@([^|].+)/) { # normal files
            my $v = $1;
            $v =~ s/\s*$//;      # strip extra space
            $v =~ s/\s*[;#].*//; # strip comments and ;
            $include{$v}=1;
        }

	if ( ($l =~ /(append|file)\s*=\s*"(.+?)"\s*=/i)
	  or
	  ($l =~ /^\s*(append|file)\s*=\s*(.+?)\s*=/i) ) {
            push @logs, $2;
            my $dirname = dirname( $2 );
            if ( ! grep { /$dirname/ } @logfolders ) {
	      push @logfolders, $dirname;
            }
        }
    }

    close($FP);
}

# we can find new config file during dump
my $security_loop=10;
while (keys %include && $security_loop-- > 0)
{
    dump_conf();
}

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering optional bacula configuration files.\n";

print "\n#============== bs_conf_opt ======================================#\n";

foreach my $f (@opt_conf) {
    my $ok = 0;
    my @fst;
    my @grinfo;
    foreach my $d (@rep) {
        if (-r "$d/$f" && !$already_seen{"$d/$f"}) {
            $already_seen{"$d/$f"}=1;
	    $fst = stat( "$d/$f" );
	    my ($fgname, $fgpasswd, $fggid, $fgmembers) = getgrgid( $fst->gid );
            print "\n#================================================================#\n";
            print "Content $d/$f- (". strftime( "%F %T", localtime( $fst->mtime ) );
            printf (", %04o - %s:", $fst->mode & 07777, getpwuid($fst->uid) );
	    print "$fgname):\n";
#	    $grinfo = getgrgid( $fst->gid );
#            printf (", %04o - %s:%s):\n", $fst->mode & 07777, getpwuid($fst->uid), $grinfo->name );
#            print "Content $d/$f- (". strftime( "%F %T", localtime( $fst->mtime ) ).", ".$fst->mode & 07777.", ". getpwuid($fst->uid).":".getgrgid( $fst->gid )."):\n";
            strip_conf("$d/$f");
            $ok++;
        }
    }
    if (!$ok) {
        print "W: Can't find $f\n";
    }
}

#BEE
# include vddk log file if any
if (-d "/opt/bweb") {
  print "\n#============== bs_conf_opt_bweb =================================#\n";
  strip_conf( "/opt/bweb/etc/bweb.conf" );
  print "\n#============== bs_conf_opt_bweb_httpd ===========================#\n";
  strip_conf( "/opt/bweb/etc/httpd.conf" );
}
if ( $sanplugin ) {
  if ( -f "/opt/bacula/scripts/storage-ctl.conf" ) {
   print "\n#============== bs_conf_sanshst ==================================#\n";
   strip_conf( "/opt/bacula/scripts/storage-ctl.conf" );
  }
}

# include vddk log file if any
if (-d "/opt/bacula/working/vsphere") {
    my @f = `find /opt/bacula/working/vsphere $findopt -name '*.log' -type f -mtime -2 2>/dev/null`;
    foreach my $file (@f) {
        chomp($file);
        push @logs, $file;
    }

    @f = `find /opt/bacula $findopt -name 'vsphere-ctl*.log' -type f -mtime -2 2>/dev/null`;
    foreach my $file (@f) {
        chomp($file);
        push @logs, $file;
    }
}

    print '#============== bs_conf_dedup_folders ==============================#', "\n";
    foreach my $i (@dedupfolders) {
      if (-d $i ) {
	print "Content of $i:\n";
	system( "ls -l ".$i );
      }
    }
#/BEE

print "\n#============== bs_conf_mtx_version ==============================#\n";
print "\nmtx:\n";
system('mtx --version 2>&1');

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula catalog backend paramaters.\n";

print "\n#============== bs_db_psql_info === pg_config ====================#\n";
print "\nPostgreSQL info:\n";
system("pg_config 2> /dev/null");
my $pq_ver = `pg_config --version 2>/dev/null`;
if ($pq_ver && $pq_ver =~ /PostgreSQL (\d+)\.(\d+)/) {
    $pq_ver = $1 * 100 + $2;
}

# if root, try with postgres user
if (!$<) {
    system('su - postgres -c "psql -l" < /dev/null 2> /dev/null');
    system('su - postgres -c "psql -c \'show all\'" 2> /dev/null');
}
system("psql -l < /dev/null 2> /dev/null");
system("psql -c 'show all' 2> /dev/null");

print "\n#============== bs_db_mysql_info === mysql_config ================#\n";
print "\nMySQL info:\n";
system("mysql_config 2> /dev/null");
system("mysql -e 'show variables;' 2> /dev/null");

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula log information.\n";

my %h = map { $_ => 1 } @logs;  # read each file only one time
foreach my $f (keys %h) {
    if (-f $f) {
        print "\n#================================================================#\n";
        print "Log content $f:\n";
        system("tail -$loglines '$f'");
    }
}
print "\n#============== bs_log_journalctl ===============================#\n";
system("journalctl --no-pager -n $loglines");
print "\n#============== bs_log_issues ===================================#\n";
if ( $#logfolders > 0 ) {
  foreach my $d ( @logfolders ) {
    print "Log content $d:\n";
#BEE
    system("grep -i ' error: append.*chunk' $d/*");
    system("grep -i ' error: .*index' $d/*");
    system("grep -i ' error: .*dde' $d/*");
    system("grep -i ' error: .*chunk' $d/*");
    system("grep -i ' error: .*FSM' $d/*");
#/BEE
  }
}
print "\n#================================================================#\n";
print "Check binaries\n";

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula job information.\n";

foreach my $f (@bin) {
    my $ok=0;
    foreach my $d (@rep) {
        if (-x "$d/$f" && !$already_seen{"$d/$f"}) {
            $already_seen{"$d/$f"}=1;
            print "\n#================================================================#\n";
            system("file '$d/$f'");
            system("md5sum '$d/$f' 2>&1");
            system("ls -l '$d/$f'");
            system("ldd '$d/$f'");
            system("$d/$f -? 2>&1 < /dev/null");
            $ok++;

            if ($f eq "bconsole" and !$already_seen{bconsole} and !$nodirinfo ) {
		my @showst = `echo "show storages"|$d/$f -n`;
		my @stnames = getStorages( @showst );
		print @stnames;
                print "Getting information from Catalog:\n";
                if (!open(FP, "|$d/$f")) {
                    delete $already_seen{"$d/$f"}; # try with the next one
                    next;
                }
		$| = 1;
                $already_seen{bconsole}=1;
                my $startweek = time() - 7*24*60*60;
                my $endweek = time();
                my $startday = time() - 2*24*60*60;
                my $endday = time() - 1*24*60*60;
                print FP "$dir_num\n";
#                print FP "gui on\n";
                print '#============== bs_conf_bee_version ==============================#', "\n";
                print FP "version\n";
                print '#============== bs_plugin_snapshots ==============================#', "\n";
                print FP "list snapshots\n";
                print FP '#============== bs_conf_status_dir ===============================#', "\n";
                print FP "status director\n";
		print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula storage information.\n";

                print FP '#============== bs_conf_status_storage ===========================#', "\n";
		if ( $skip_storage ) {
		    print FP 'Data collection disabled by user\n';
		}
		else {
		  foreach my $sdnames (@stnames) {
		    print FP "status storage=\"$sdnames\"\n";
		  }
		}
                print FP '#============== bs_conf_dedup_usage ==============================#', "\n";
		if ( $skip_storage ) {
		    print FP 'Data collection disabled by user\n';
		}
		else {
		  foreach my $sdnames (@stnames) {
		    print FP "dedup usage storage=\"$sdnames\"\n";
		  }
		}
		print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula active information.\n";
                print FP '#============== bs_conf_list_pools ===============================#', "\n";
                print FP "list pools\n";
                print FP '#============== bs_conf_list_media ===============================#', "\n";
                print FP "list media\n";
                print FP '#============== bs_conf_list_jobs ================================#', "\n";
                print FP "list jobs\n";
                print FP '#============== bs_conf_show_jobs ================================#', "\n";
                print FP "show job\n";
                print FP '#============== bs_sql_pseudo ====================================#', "\n";
                print FP "sql\n";
                print "\n#============== bs_db_psql_info ==================================#\n";
                print FP "show all;\n";
                print "\n#============== bs_db_mysql_info =================================#\n";
                print FP "show variables;\n";

		print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula catalog statistics.\n";

                # display indexes for mysql and pg
#            print "\n#================================================================#;\n";
                print FP "SHOW INDEXES FROM File;\n";
#            print "\n#================================================================#;\n";
                print FP "SELECT c.relname as Name,",
" c2.relname as Table,",
" pg_catalog.pg_get_userbyid(c.relowner) as Owner,",
" pg_catalog.pg_size_pretty(pg_catalog.pg_relation_size(c.oid)) as Size ",
"FROM pg_catalog.pg_class c",
"     LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace",
"     LEFT JOIN pg_catalog.pg_index i ON i.indexrelid = c.oid",
"     LEFT JOIN pg_catalog.pg_class c2 ON i.indrelid = c2.oid ",
"WHERE c.relkind IN ('i','')",
"      AND n.nspname <> 'pg_catalog'",
"      AND n.nspname <> 'information_schema'",
"      AND n.nspname !~ '^pg_toast'",
"  AND pg_catalog.pg_table_is_visible(c.oid) ",
"ORDER BY 1,2;\n";
	    print FP "-- #============== bs_db_table_stat =================================#;\n";
	    print FP "select * from pg_stat_user_tables;\n";
                if ($pq_ver && $pq_ver < 900) {
                    print FP "SELECT 'ERROR: problem on pq/pg version, must upgrade local libpq' ",
                               "FROM (SELECT version() as version) as a ", 
                              "WHERE version LIKE 'PostgreSQL 9%';\n";
                }
            print FP "-- #============== bs_db_version ====================================#;\n";
                print FP "SELECT version();\n";
                # get some statistics
            print FP "-- #============== bs_cat_totals ====================================#;\n";
                print FP "SELECT Type, COUNT(1) AS TotalJobNumber,",
                                "SUM(JobBytes)/1024/1024/1024 AS TotalSumGB,",
                                "SUM(JobFiles) AS TotalSumFiles FROM Job WHERE Type IN ('B','C') ",
                                "AND JobStatus = 'T' GROUP BY Type;\n";
            print FP "-- #============== bs_cat_weekly ====================================#;\n";
                print FP "SELECT Type, COUNT(1) AS WeekJobNumber,",
                                "SUM(JobBytes)/1024/1024/1024 AS WeekTotalSumGB,",
                                "SUM(JobFiles) AS WeekTotalSumFiles FROM Job ",
                                "WHERE Type IN ('B','C') AND JobStatus = 'T' AND JobTDate > $startweek ",
                                  "AND JobTDate < $endweek GROUP BY Type;\n";
            print FP "-- #============== bs_cat_daily =====================================#;\n";
                print FP "SELECT Type, COUNT(1) AS DayJobNumber,",
                                "SUM(JobBytes)/1024/1024/1024 AS DayTotalSumGB,",
                                "SUM(JobFiles) AS DayTotalSumFiles FROM Job ",
                                "WHERE Type IN ('B','C') AND JobStatus = 'T' ",
                                  "AND JobTDate > $startday AND JobTDate < $endday GROUP BY Type;\n";
            print FP "-- #============== bs_cat_clients ===================================#;\n";
                print FP "SELECT Name, Uname FROM Client;\n";
            print FP "-- #============== bs_cat_clients_uname =============================#;\n";
                print FP "SELECT Count(1), Uname FROM Client GROUP BY Uname;\n";
            print FP "-- #============== bs_cat_client_sum ================================#;\n";
                print FP "SELECT Count(1) FROM Client;\n";
            print FP "-- #============== bs_cat_count_logs ======================================#;\n";
                print FP "SELECT Count(1) FROM Log;\n";
            print FP "-- #============== bs_cat_count_filenames =================================#;\n";
# Mysql estimate queries
#		print FP "SELECT table_rows FROM information_schema.tables WHERE table_name=Filename AND table_schema=bacula;\n";
# Postgres estimate queries
# SELECT (reltuples/relpages) * (pg_relation_size('File') / (current_setting('block_size')::integer) ) FROM pg_class where relname = 'File';
                print FP "SELECT Count(1) FROM Filename;\n";
                print FP "SELECT Count(1) FROM File;\n";
           print FP "-- #============== bs_cat_paths =====================================#;\n";
                print FP "SELECT Count(1) FROM Path;\n";
            print FP "-- #============== bs_cat_volume_files ==============================#;\n";
                print FP "SELECT Count(1), MAX(VolBytes), MAX(VolFiles) FROM Media;\n";
            print FP "-- #============== bs_active_clients ================================#;\n";
# PGSQL
#              print FP "select count(distinct clientid) as ActiveClients from job where starttime > CURRENT_DATE - INTERVAL '$daysback day' and type='B';\n";
#            print FP "select count(distinct clientid) as ActiveClients from job where starttime > CURRENT_DATE - INTERVAL '$daysback day' and type='B'\n";
#	     print FP "select (select distinct client.name,job.jobbytes from client,job where client.clientid=job.clientid and starttime > CURRENT_DATE - INTERVAL '$daysback day' and type='B') as \"Active Clients\", round(sum(cast(jobbytes as numeric)/1024/1024/1024),3) as \"GBytes Used\" from job group by \"Active Clients\" order by \"GBytes Used\" desc;";
	     print FP "select distinct client.name as \"Active Clients\" from client,job where client.clientid=job.clientid and job.type='B'and job.starttime > CURRENT_DATE - INTERVAL '$daysback days';\n";
	     print FP "select count (distinct clientid) as \"Active Clients\" from job where job.type='B'and job.starttime > CURRENT_DATE - INTERVAL '$daysback days';\n";
# MySQL
             print FP " select distinct Client.Name as ActiveClients from Client,Job where Client.ClientId=Job.ClientId and Job.StartTime >DATE_ADD(curdate(),INTERVAL -$daysback DAY) and Job.Type='B';\n";
             print FP "select count(distinct ClientId) as ActiveClients from Job where StartTime >DATE_ADD(curdate(),INTERVAL -$daysback DAY) and Type='B';\n";
#BEE
#select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Hyper-V';
#select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Hyper-V';
#select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='vSphere';
#select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='vSphere';
#select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Proxmox';
#select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Proxmox';
#select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='RHV';
#select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='RHV';

	    if ( $vsphereplugin == 1 ) {
	      print FP "-- #============== bs_vsphere_clients ===============================#;\n";
	      print FP "SELECT count(distinct Filename) FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@vsphere/';\n";
	      print FP "SELECT count(distinct Path) from Path where Path LIKE '/\@vsphere/%/';\n";
	      print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@vsphere/';\n";
	      print FP "SELECT distinct Path from Path where Path LIKE '/\@vsphere/%/';\n";
	      print FP "-- Get Vsphere from the objectname table;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='vSphere';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='vSphere';\n";
	    }
	    print "\n";
	    if ( $hypervplugin == 1 ) {
	      print FP "-- #============== bs_hyper-v_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype LIKE '%Hyper-V';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype LIKE '%Hyper-V';\n";
	    }
	    print "\n";
	    if ( $rhvplugin == 1 ) {
	      print FP "-- #============== bs_rhv_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='rhv';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='rhv';\n";
	    }
	    print "\n";
	    if ( $proxmoxplugin == 1 ) {
	      print FP "-- #============== bs_proxmox_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='proxmox';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='proxmox';\n";
	    }
	    if ( $xenserverplugin == 1 ) {
	      print FP "-- #============== bs_XEN_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='xen';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='xen';\n";
	    }
	    print "\n";
	    if ( $nutanixplugin == 1 ) {
	      print FP "-- #============== bs_nutanix_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='nutanix-ahv';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='nutanix-ahv';\n";
	    }
	    print "\n";
	    if ( $qemuplugin == 1 ) {
	      print FP "-- #============== bs_rhv_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='qemu';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='qemu';\n";
	    }
	    print "\n";
	    if ( $azureplugin == 1 ) {
	      print FP "-- #============== bs_rhv_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='azure-vm';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='azure-vm';\n";
	    }
	    print "\n";
	    if ( $ec2plugin == 1 ) {
	      print FP "-- #============== bs_rhv_clients ===============================#;\n";
	      print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='amazon-ec2';\n";
	      print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='amazon-ec2';\n";
	    }
	    print "\n";
	    print FP "-- #============== bs_beta_info =====================================#;\n";
	    print FP "-- HYPER-V;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@HYPERV/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@HYPERV/%/';\n";
	    print FP "-- RHEV;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@rhv/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@rhv/%/';\n";
	    print FP "-- ProxMox;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@proxmox/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@proxmox/%/';\n";
	    print FP "-- XEN;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@xen/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@xen/%/';\n";
	    print FP "-- Nutanix;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@nutanix-ahv/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@nutanix-ahv/%/';\n";
	    print FP "-- QEMU;\n";
	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@qemu/';\n";
	    print FP "SELECT distinct Path from Path where Path LIKE '/\@qemu/%/';\n";

	    print FP "-- kubernetes;\n";
	    print FP "SELECT count(distinct File.FileName) FROM Path INNER JOIN File ON Path.Pathid=File.Pathid WHERE Path.path LIKE '/\@kubernetes%persistentvolumeclaims%' AND filename LIKE '%yaml';\n";
	    print FP "-- Get VM information from the objectname table;\n";
	    print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Hyper-V';\n";
	    print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Hyper-V';\n";
	    print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Proxmox';\n";
	    print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='Proxmox';\n";
	    print FP "select distinct(objectname) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='RHV';\n";
	    print FP "select count(distinct(objectname)) from Object where objectcategory='Virtual Machine' and objectname<>'' and objecttype='RHV';\n";

#	    print FP "SELECT distinct Filename FROM File JOIN Path USING (PathId) where Path.Path LIKE '/\@kubernetes/';\n";
#	    print FP "SELECT distinct Path from Path where Path LIKE '/\@kubernetes/%/';\n";
#/BEE

	    $| = 0;
            close(FP);
          }
        }
    }
    if (!$ok) {
        print "W: Can't find $f\n";
    }
    if ($ok > 1) {
        if ( $f != 'bconsole' ) {
#	  print STDERR "Found multiple instances of $f\n";
	  print "W: Found multiple instances of $f\n";
        }
    }
}
print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering SAR information from the server.\n";

print "\n#============== bs_sar ===========================================#\n";
print "\nSAR:\n";
system("sar -A");

print STDERR strftime( "%a %b %e %H:%M:%S %Y", localtime )." - Gathering bacula debug information.\n";

print "\n#============== bs_bee_traces ====================================#\n";
my @places;
# display btrace files
foreach my $f (@bin) {
    foreach my $d (@rep) {
        if (-d $d) {
            print "btrace $d:\n";
            push @places , `find '$d' $findopt -name '*.traceback' -ls 2>&1`;
        }
    }
}

%h = map { $_ => 1 } @places;
print keys %h;

close(STDOUT);
